/**
 * @tanstack/router-core/src/index.ts
 *
 * Copyright (c) TanStack
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.md file in the root directory of this source tree.
 *
 * @license MIT
 */
(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('use-sync-external-store/shim/with-selector')) :
    typeof define === 'function' && define.amd ? define(['exports', 'use-sync-external-store/shim/with-selector'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.RouterCore = {}));
})(this, (function (exports) { 'use strict';

    var prefix = 'Invariant failed';
    function invariant(condition, message) {
        if (condition) {
            return;
        }
        var provided = typeof message === 'function' ? message() : message;
        var value = provided ? "".concat(prefix, ": ").concat(provided) : prefix;
        throw new Error(value);
    }

    function warning(condition, message) {
      {
        if (condition) {
          return;
        }

        var text = "Warning: " + message;

        if (typeof console !== 'undefined') {
          console.warn(text);
        }

        try {
          throw Error(text);
        } catch (x) {}
      }
    }

    // While the public API was clearly inspired by the "history" npm package,
    // This implementation attempts to be more lightweight by
    // making assumptions about the way TanStack Router works

    const pushStateEvent = 'pushstate';
    const popStateEvent = 'popstate';
    const beforeUnloadEvent = 'beforeunload';
    const beforeUnloadListener = event => {
      event.preventDefault();
      // @ts-ignore
      return event.returnValue = '';
    };
    const stopBlocking = () => {
      removeEventListener(beforeUnloadEvent, beforeUnloadListener, {
        capture: true
      });
    };
    function createHistory(opts) {
      let location = opts.getLocation();
      let unsub = () => {};
      let listeners = new Set();
      let blockers = [];
      let queue = [];
      const tryFlush = () => {
        if (blockers.length) {
          blockers[0]?.(tryFlush, () => {
            blockers = [];
            stopBlocking();
          });
          return;
        }
        while (queue.length) {
          queue.shift()?.();
        }
        if (!opts.listener) {
          onUpdate();
        }
      };
      const queueTask = task => {
        queue.push(task);
        tryFlush();
      };
      const onUpdate = () => {
        location = opts.getLocation();
        listeners.forEach(listener => listener());
      };
      return {
        get location() {
          return location;
        },
        listen: cb => {
          if (listeners.size === 0) {
            unsub = typeof opts.listener === 'function' ? opts.listener(onUpdate) : () => {};
          }
          listeners.add(cb);
          return () => {
            listeners.delete(cb);
            if (listeners.size === 0) {
              unsub();
            }
          };
        },
        push: (path, state) => {
          queueTask(() => {
            opts.pushState(path, state);
          });
        },
        replace: (path, state) => {
          queueTask(() => {
            opts.replaceState(path, state);
          });
        },
        go: index => {
          queueTask(() => {
            opts.go(index);
          });
        },
        back: () => {
          queueTask(() => {
            opts.back();
          });
        },
        forward: () => {
          queueTask(() => {
            opts.forward();
          });
        },
        createHref: str => opts.createHref(str),
        block: cb => {
          blockers.push(cb);
          if (blockers.length === 1) {
            addEventListener(beforeUnloadEvent, beforeUnloadListener, {
              capture: true
            });
          }
          return () => {
            blockers = blockers.filter(b => b !== cb);
            if (!blockers.length) {
              stopBlocking();
            }
          };
        }
      };
    }
    function createBrowserHistory(opts) {
      const getHref = opts?.getHref ?? (() => `${window.location.pathname}${window.location.search}${window.location.hash}`);
      const createHref = opts?.createHref ?? (path => path);
      const getLocation = () => parseLocation(getHref(), history.state);
      return createHistory({
        getLocation,
        listener: onUpdate => {
          window.addEventListener(pushStateEvent, onUpdate);
          window.addEventListener(popStateEvent, onUpdate);
          var pushState = window.history.pushState;
          window.history.pushState = function () {
            let res = pushState.apply(history, arguments);
            onUpdate();
            return res;
          };
          var replaceState = window.history.replaceState;
          window.history.replaceState = function () {
            let res = replaceState.apply(history, arguments);
            onUpdate();
            return res;
          };
          return () => {
            window.history.pushState = pushState;
            window.history.replaceState = replaceState;
            window.removeEventListener(pushStateEvent, onUpdate);
            window.removeEventListener(popStateEvent, onUpdate);
          };
        },
        pushState: (path, state) => {
          window.history.pushState({
            ...state,
            key: createRandomKey()
          }, '', createHref(path));
        },
        replaceState: (path, state) => {
          window.history.replaceState({
            ...state,
            key: createRandomKey()
          }, '', createHref(path));
        },
        back: () => window.history.back(),
        forward: () => window.history.forward(),
        go: n => window.history.go(n),
        createHref: path => createHref(path)
      });
    }
    function createHashHistory() {
      return createBrowserHistory({
        getHref: () => window.location.hash.substring(1),
        createHref: path => `#${path}`
      });
    }
    function createMemoryHistory(opts = {
      initialEntries: ['/']
    }) {
      const entries = opts.initialEntries;
      let index = opts.initialIndex ?? entries.length - 1;
      let currentState = {};
      const getLocation = () => parseLocation(entries[index], currentState);
      return createHistory({
        getLocation,
        listener: false,
        pushState: (path, state) => {
          currentState = {
            ...state,
            key: createRandomKey()
          };
          entries.push(path);
          index++;
        },
        replaceState: (path, state) => {
          currentState = {
            ...state,
            key: createRandomKey()
          };
          entries[index] = path;
        },
        back: () => {
          index--;
        },
        forward: () => {
          index = Math.min(index + 1, entries.length - 1);
        },
        go: n => window.history.go(n),
        createHref: path => path
      });
    }
    function parseLocation(href, state) {
      let hashIndex = href.indexOf('#');
      let searchIndex = href.indexOf('?');
      return {
        href,
        pathname: href.substring(0, hashIndex > 0 ? searchIndex > 0 ? Math.min(hashIndex, searchIndex) : hashIndex : searchIndex > 0 ? searchIndex : href.length),
        hash: hashIndex > -1 ? href.substring(hashIndex) : '',
        search: searchIndex > -1 ? href.slice(searchIndex, hashIndex === -1 ? undefined : hashIndex) : '',
        state
      };
    }

    // Thanks co-pilot!
    function createRandomKey() {
      return (Math.random() + 1).toString(36).substring(7);
    }

    function last(arr) {
      return arr[arr.length - 1];
    }
    function isFunction(d) {
      return typeof d === 'function';
    }
    function functionalUpdate(updater, previous) {
      if (isFunction(updater)) {
        return updater(previous);
      }
      return updater;
    }
    function pick(parent, keys) {
      return keys.reduce((obj, key) => {
        obj[key] = parent[key];
        return obj;
      }, {});
    }

    /**
     * This function returns `a` if `b` is deeply equal.
     * If not, it will replace any deeply equal children of `b` with those of `a`.
     * This can be used for structural sharing between immutable JSON values for example.
     * Do not use this with signals
     */
    function replaceEqualDeep(prev, _next) {
      if (prev === _next) {
        return prev;
      }
      const next = _next;
      const array = Array.isArray(prev) && Array.isArray(next);
      if (array || isPlainObject(prev) && isPlainObject(next)) {
        const prevSize = array ? prev.length : Object.keys(prev).length;
        const nextItems = array ? next : Object.keys(next);
        const nextSize = nextItems.length;
        const copy = array ? [] : {};
        let equalItems = 0;
        for (let i = 0; i < nextSize; i++) {
          const key = array ? i : nextItems[i];
          copy[key] = replaceEqualDeep(prev[key], next[key]);
          if (copy[key] === prev[key]) {
            equalItems++;
          }
        }
        return prevSize === nextSize && equalItems === prevSize ? prev : copy;
      }
      return next;
    }

    // Copied from: https://github.com/jonschlinkert/is-plain-object
    function isPlainObject(o) {
      if (!hasObjectPrototype(o)) {
        return false;
      }

      // If has modified constructor
      const ctor = o.constructor;
      if (typeof ctor === 'undefined') {
        return true;
      }

      // If has modified prototype
      const prot = ctor.prototype;
      if (!hasObjectPrototype(prot)) {
        return false;
      }

      // If constructor does not have an Object-specific method
      if (!prot.hasOwnProperty('isPrototypeOf')) {
        return false;
      }

      // Most likely a plain Object
      return true;
    }
    function hasObjectPrototype(o) {
      return Object.prototype.toString.call(o) === '[object Object]';
    }
    function partialDeepEqual(a, b) {
      if (a === b) {
        return true;
      }
      if (typeof a !== typeof b) {
        return false;
      }
      if (isPlainObject(a) && isPlainObject(b)) {
        return !Object.keys(b).some(key => !partialDeepEqual(a[key], b[key]));
      }
      if (Array.isArray(a) && Array.isArray(b)) {
        return a.length === b.length && a.every((item, index) => partialDeepEqual(item, b[index]));
      }
      return false;
    }

    function joinPaths(paths) {
      return cleanPath(paths.filter(Boolean).join('/'));
    }
    function cleanPath(path) {
      // remove double slashes
      return path.replace(/\/{2,}/g, '/');
    }
    function trimPathLeft(path) {
      return path === '/' ? path : path.replace(/^\/{1,}/, '');
    }
    function trimPathRight(path) {
      return path === '/' ? path : path.replace(/\/{1,}$/, '');
    }
    function trimPath(path) {
      return trimPathRight(trimPathLeft(path));
    }
    function resolvePath(basepath, base, to) {
      base = base.replace(new RegExp(`^${basepath}`), '/');
      to = to.replace(new RegExp(`^${basepath}`), '/');
      let baseSegments = parsePathname(base);
      const toSegments = parsePathname(to);
      toSegments.forEach((toSegment, index) => {
        if (toSegment.value === '/') {
          if (!index) {
            // Leading slash
            baseSegments = [toSegment];
          } else if (index === toSegments.length - 1) {
            // Trailing Slash
            baseSegments.push(toSegment);
          } else ;
        } else if (toSegment.value === '..') {
          // Extra trailing slash? pop it off
          if (baseSegments.length > 1 && last(baseSegments)?.value === '/') {
            baseSegments.pop();
          }
          baseSegments.pop();
        } else if (toSegment.value === '.') {
          return;
        } else {
          baseSegments.push(toSegment);
        }
      });
      const joined = joinPaths([basepath, ...baseSegments.map(d => d.value)]);
      return cleanPath(joined);
    }
    function parsePathname(pathname) {
      if (!pathname) {
        return [];
      }
      pathname = cleanPath(pathname);
      const segments = [];
      if (pathname.slice(0, 1) === '/') {
        pathname = pathname.substring(1);
        segments.push({
          type: 'pathname',
          value: '/'
        });
      }
      if (!pathname) {
        return segments;
      }

      // Remove empty segments and '.' segments
      const split = pathname.split('/').filter(Boolean);
      segments.push(...split.map(part => {
        if (part === '$' || part === '*') {
          return {
            type: 'wildcard',
            value: part
          };
        }
        if (part.charAt(0) === '$') {
          return {
            type: 'param',
            value: part
          };
        }
        return {
          type: 'pathname',
          value: part
        };
      }));
      if (pathname.slice(-1) === '/') {
        pathname = pathname.substring(1);
        segments.push({
          type: 'pathname',
          value: '/'
        });
      }
      return segments;
    }
    function interpolatePath(path, params, leaveWildcards = false) {
      const interpolatedPathSegments = parsePathname(path);
      return joinPaths(interpolatedPathSegments.map(segment => {
        if (segment.type === 'wildcard') {
          const value = params[segment.value];
          if (leaveWildcards) return `${segment.value}${value ?? ''}`;
          return value;
        }
        if (segment.type === 'param') {
          return params[segment.value.substring(1)] ?? '';
        }
        return segment.value;
      }));
    }
    function matchPathname(basepath, currentPathname, matchLocation) {
      const pathParams = matchByPath(basepath, currentPathname, matchLocation);
      // const searchMatched = matchBySearch(location.search, matchLocation)

      if (matchLocation.to && !pathParams) {
        return;
      }
      return pathParams ?? {};
    }
    function matchByPath(basepath, from, matchLocation) {
      // Remove the base path from the pathname
      from = basepath != '/' ? from.substring(basepath.length) : from;
      // Default to to $ (wildcard)
      const to = `${matchLocation.to ?? '$'}`;
      // Parse the from and to
      const baseSegments = parsePathname(from);
      const routeSegments = parsePathname(to);
      if (!from.startsWith('/')) {
        baseSegments.unshift({
          type: 'pathname',
          value: '/'
        });
      }
      if (!to.startsWith('/')) {
        routeSegments.unshift({
          type: 'pathname',
          value: '/'
        });
      }
      const params = {};
      let isMatch = (() => {
        for (let i = 0; i < Math.max(baseSegments.length, routeSegments.length); i++) {
          const baseSegment = baseSegments[i];
          const routeSegment = routeSegments[i];
          const isLastBaseSegment = i >= baseSegments.length - 1;
          const isLastRouteSegment = i >= routeSegments.length - 1;
          if (routeSegment) {
            if (routeSegment.type === 'wildcard') {
              if (baseSegment?.value) {
                params['*'] = joinPaths(baseSegments.slice(i).map(d => d.value));
                return true;
              }
              return false;
            }
            if (routeSegment.type === 'pathname') {
              if (routeSegment.value === '/' && !baseSegment?.value) {
                return true;
              }
              if (baseSegment) {
                if (matchLocation.caseSensitive) {
                  if (routeSegment.value !== baseSegment.value) {
                    return false;
                  }
                } else if (routeSegment.value.toLowerCase() !== baseSegment.value.toLowerCase()) {
                  return false;
                }
              }
            }
            if (!baseSegment) {
              return false;
            }
            if (routeSegment.type === 'param') {
              if (baseSegment?.value === '/') {
                return false;
              }
              if (baseSegment.value.charAt(0) !== '$') {
                params[routeSegment.value.substring(1)] = baseSegment.value;
              }
            }
          }
          if (!isLastBaseSegment && isLastRouteSegment) {
            return !!matchLocation.fuzzy;
          }
        }
        return true;
      })();
      return isMatch ? params : undefined;
    }

    // @ts-nocheck

    // qss has been slightly modified and inlined here for our use cases (and compression's sake). We've included it as a hard dependency for MIT license attribution.

    function encode(obj, pfx) {
      var k,
        i,
        tmp,
        str = '';
      for (k in obj) {
        if ((tmp = obj[k]) !== void 0) {
          if (Array.isArray(tmp)) {
            for (i = 0; i < tmp.length; i++) {
              str && (str += '&');
              str += encodeURIComponent(k) + '=' + encodeURIComponent(tmp[i]);
            }
          } else {
            str && (str += '&');
            str += encodeURIComponent(k) + '=' + encodeURIComponent(tmp);
          }
        }
      }
      return (pfx || '') + str;
    }
    function toValue(mix) {
      if (!mix) return '';
      var str = decodeURIComponent(mix);
      if (str === 'false') return false;
      if (str === 'true') return true;
      return +str * 0 === 0 && +str + '' === str ? +str : str;
    }
    function decode(str) {
      var tmp,
        k,
        out = {},
        arr = str.split('&');
      while (tmp = arr.shift()) {
        tmp = tmp.split('=');
        k = tmp.shift();
        if (out[k] !== void 0) {
          out[k] = [].concat(out[k], toValue(tmp.shift()));
        } else {
          out[k] = toValue(tmp.shift());
        }
      }
      return out;
    }

    const rootRouteId = '__root__';

    // | ParseParamsObj<TPath, TParams>

    // The parse type here allows a zod schema to be passed directly to the validator

    class Route {
      // Set up in this.init()

      // customId!: TCustomId

      // Optional

      constructor(options) {
        this.options = options || {};
        this.isRoot = !options?.getParentRoute;
        Route.__onInit(this);
      }
      init = opts => {
        this.originalIndex = opts.originalIndex;
        this.router = opts.router;
        const options = this.options;
        const isRoot = !options?.path && !options?.id;
        this.parentRoute = this.options?.getParentRoute?.();
        if (isRoot) {
          this.path = rootRouteId;
        } else {
          invariant(this.parentRoute, `Child Route instances must pass a 'getParentRoute: () => ParentRoute' option that returns a Route instance.`);
        }
        let path = isRoot ? rootRouteId : options.path;

        // If the path is anything other than an index path, trim it up
        if (path && path !== '/') {
          path = trimPath(path);
        }
        const customId = options?.id || path;

        // Strip the parentId prefix from the first level of children
        let id = isRoot ? rootRouteId : joinPaths([this.parentRoute.id === rootRouteId ? '' : this.parentRoute.id, customId]);
        if (path === rootRouteId) {
          path = '/';
        }
        if (id !== rootRouteId) {
          id = joinPaths(['/', id]);
        }
        const fullPath = id === rootRouteId ? '/' : joinPaths([this.parentRoute.fullPath, path]);
        this.path = path;
        this.id = id;
        // this.customId = customId as TCustomId
        this.fullPath = fullPath;
        this.to = fullPath;
      };
      addChildren = children => {
        this.children = children;
        return this;
      };
      update = options => {
        Object.assign(this.options, options);
        return this;
      };
      static __onInit = route => {
        // This is a dummy static method that should get
        // replaced by a framework specific implementation if necessary
      };
    }
    class RouterContext {
      constructor() {}
      createRootRoute = options => {
        return new RootRoute(options);
      };
    }
    class RootRoute extends Route {
      constructor(options) {
        super(options);
      }
    }

    class FileRoute {
      constructor(path) {
        this.path = path;
      }
      createRoute = options => {
        const route = new Route(options);
        route.isRoot = false;
        return route;
      };
    }

    /**
     * @tanstack/store/src/index.ts
     *
     * Copyright (c) TanStack
     *
     * This source code is licensed under the MIT license found in the
     * LICENSE.md file in the root directory of this source tree.
     *
     * @license MIT
     */
    class Store {
      listeners = new Set();
      _batching = false;
      _flushing = 0;
      _nextPriority = null;
      constructor(initialState, options) {
        this.state = initialState;
        this.options = options;
      }
      subscribe = listener => {
        this.listeners.add(listener);
        const unsub = this.options?.onSubscribe?.(listener, this);
        return () => {
          this.listeners.delete(listener);
          unsub?.();
        };
      };
      setState = (updater, opts) => {
        const previous = this.state;
        this.state = this.options?.updateFn ? this.options.updateFn(previous)(updater) : updater(previous);
        const priority = opts?.priority ?? this.options?.defaultPriority ?? 'high';
        if (this._nextPriority === null) {
          this._nextPriority = priority;
        } else if (this._nextPriority === 'high') {
          this._nextPriority = priority;
        } else {
          this._nextPriority = this.options?.defaultPriority ?? 'high';
        }

        // Always run onUpdate, regardless of batching
        this.options?.onUpdate?.({
          priority: this._nextPriority
        });

        // Attempt to flush
        this._flush();
      };
      _flush = () => {
        if (this._batching) return;
        const flushId = ++this._flushing;
        this.listeners.forEach(listener => {
          if (this._flushing !== flushId) return;
          listener({
            priority: this._nextPriority ?? 'high'
          });
        });
      };
      batch = cb => {
        if (this._batching) return cb();
        this._batching = true;
        cb();
        this._batching = false;
        this._flush();
      };
    }

    const defaultParseSearch = parseSearchWith(JSON.parse);
    const defaultStringifySearch = stringifySearchWith(JSON.stringify);
    function parseSearchWith(parser) {
      return searchStr => {
        if (searchStr.substring(0, 1) === '?') {
          searchStr = searchStr.substring(1);
        }
        let query = decode(searchStr);

        // Try to parse any query params that might be json
        for (let key in query) {
          const value = query[key];
          if (typeof value === 'string') {
            try {
              query[key] = parser(value);
            } catch (err) {
              //
            }
          }
        }
        return query;
      };
    }
    function stringifySearchWith(stringify) {
      return search => {
        search = {
          ...search
        };
        if (search) {
          Object.keys(search).forEach(key => {
            const val = search[key];
            if (typeof val === 'undefined' || val === undefined) {
              delete search[key];
            } else if (val && typeof val === 'object' && val !== null) {
              try {
                search[key] = stringify(val);
              } catch (err) {
                // silent
              }
            }
          });
        }
        const searchStr = encode(search).toString();
        return searchStr ? `?${searchStr}` : '';
      };
    }

    //

    const componentTypes = ['component', 'errorComponent', 'pendingComponent'];
    class Router {
      #unsubHistory;
      constructor(options) {
        this.options = {
          defaultPreloadDelay: 50,
          context: undefined,
          ...options,
          stringifySearch: options?.stringifySearch ?? defaultStringifySearch,
          parseSearch: options?.parseSearch ?? defaultParseSearch
          // fetchServerDataFn: options?.fetchServerDataFn ?? defaultFetchServerDataFn,
        };

        this.__store = new Store(getInitialRouterState(), {
          onUpdate: () => {
            const prev = this.state;
            this.state = this.__store.state;
            const matchesByIdChanged = prev.matchesById !== this.state.matchesById;
            let matchesChanged;
            let pendingMatchesChanged;
            if (!matchesByIdChanged) {
              matchesChanged = prev.matchIds.length !== this.state.matchIds.length || prev.matchIds.some((d, i) => d !== this.state.matchIds[i]);
              pendingMatchesChanged = prev.pendingMatchIds.length !== this.state.pendingMatchIds.length || prev.pendingMatchIds.some((d, i) => d !== this.state.pendingMatchIds[i]);
            }
            if (matchesByIdChanged || matchesChanged) {
              this.state.matches = this.state.matchIds.map(id => {
                return this.state.matchesById[id];
              });
            }
            if (matchesByIdChanged || pendingMatchesChanged) {
              this.state.pendingMatches = this.state.pendingMatchIds.map(id => {
                return this.state.matchesById[id];
              });
            }
            this.state.isFetching = [...this.state.matches, ...this.state.pendingMatches].some(d => d.isFetching);
          },
          defaultPriority: 'low'
        });
        this.state = this.__store.state;
        this.update(options);
        const next = this.buildNext({
          hash: true,
          fromCurrent: true,
          search: true,
          state: true
        });
        if (this.state.location.href !== next.href) {
          this.#commitLocation({
            ...next,
            replace: true
          });
        }
      }
      reset = () => {
        this.__store.setState(s => Object.assign(s, getInitialRouterState()));
      };
      mount = () => {
        // If the router matches are empty, start loading the matches
        // if (!this.state.matches.length) {
        this.safeLoad();
        // }
      };

      update = opts => {
        this.options = {
          ...this.options,
          ...opts,
          context: {
            ...this.options.context,
            ...opts?.context
          }
        };
        if (!this.history || this.options.history && this.options.history !== this.history) {
          if (this.#unsubHistory) {
            this.#unsubHistory();
          }
          this.history = this.options.history ?? (isServer ? createMemoryHistory() : createBrowserHistory());
          const parsedLocation = this.#parseLocation();
          this.__store.setState(s => ({
            ...s,
            resolvedLocation: parsedLocation,
            location: parsedLocation
          }));
          this.#unsubHistory = this.history.listen(() => {
            this.safeLoad({
              next: this.#parseLocation(this.state.location)
            });
          });
        }
        const {
          basepath,
          routeTree
        } = this.options;
        this.basepath = `/${trimPath(basepath ?? '') ?? ''}`;
        if (routeTree && routeTree !== this.routeTree) {
          this.#buildRouteTree(routeTree);
        }
        return this;
      };
      buildNext = opts => {
        const next = this.#buildLocation(opts);
        const __matches = this.matchRoutes(next.pathname, next.search);
        return this.#buildLocation({
          ...opts,
          __matches
        });
      };
      cancelMatches = () => {
        this.state.matches.forEach(match => {
          this.cancelMatch(match.id);
        });
      };
      cancelMatch = id => {
        this.getRouteMatch(id)?.abortController?.abort();
      };
      safeLoad = opts => {
        return this.load(opts).catch(err => {
          // console.warn(err)
          // invariant(false, 'Encountered an error during router.load()! ☝️.')
        });
      };
      latestLoadPromise = Promise.resolve();
      load = async opts => {
        const promise = new Promise(async (resolve, reject) => {
          let latestPromise;
          const checkLatest = () => {
            return this.latestLoadPromise !== promise ? this.latestLoadPromise : undefined;
          };

          // Cancel any pending matches
          // this.cancelMatches()

          let pendingMatches;
          this.__store.batch(() => {
            if (opts?.next) {
              // Ingest the new location
              this.__store.setState(s => ({
                ...s,
                location: opts.next
              }));
            }

            // Match the routes
            pendingMatches = this.matchRoutes(this.state.location.pathname, this.state.location.search, {
              throwOnError: opts?.throwOnError,
              debug: true
            });
            this.__store.setState(s => ({
              ...s,
              status: 'pending',
              pendingMatchIds: pendingMatches.map(d => d.id),
              matchesById: this.#mergeMatches(s.matchesById, pendingMatches)
            }));
          });
          try {
            // Load the matches
            await this.loadMatches(pendingMatches);

            // Only apply the latest transition
            if (latestPromise = checkLatest()) {
              return await latestPromise;
            }
            const prevLocation = this.state.resolvedLocation;
            this.__store.setState(s => ({
              ...s,
              status: 'idle',
              resolvedLocation: s.location,
              matchIds: s.pendingMatchIds,
              pendingMatchIds: []
            }));
            if (prevLocation.href !== this.state.location.href) {
              this.options.onRouteChange?.();
            }
            resolve();
          } catch (err) {
            // Only apply the latest transition
            if (latestPromise = checkLatest()) {
              return await latestPromise;
            }
            reject(err);
          }
        });
        this.latestLoadPromise = promise;
        return this.latestLoadPromise;
      };
      #mergeMatches = (prevMatchesById, nextMatches) => {
        const nextMatchesById = {
          ...prevMatchesById
        };
        let hadNew = false;
        nextMatches.forEach(match => {
          if (!nextMatchesById[match.id]) {
            hadNew = true;
            nextMatchesById[match.id] = match;
          }
        });
        if (!hadNew) {
          return prevMatchesById;
        }
        return nextMatchesById;
      };
      getRoute = id => {
        const route = this.routesById[id];
        invariant(route, `Route with id "${id}" not found`);
        return route;
      };
      preloadRoute = async (navigateOpts = this.state.location) => {
        const next = this.buildNext(navigateOpts);
        const matches = this.matchRoutes(next.pathname, next.search, {
          throwOnError: true
        });
        this.__store.setState(s => {
          return {
            ...s,
            matchesById: this.#mergeMatches(s.matchesById, matches)
          };
        });
        await this.loadMatches(matches, {
          preload: true,
          maxAge: navigateOpts.maxAge
        });
        return matches;
      };
      cleanMatches = () => {
        const now = Date.now();
        const outdatedMatchIds = Object.values(this.state.matchesById).filter(match => {
          const route = this.getRoute(match.routeId);
          return !this.state.matchIds.includes(match.id) && !this.state.pendingMatchIds.includes(match.id) && match.preloadInvalidAt < now && (route.options.gcMaxAge ? match.updatedAt + route.options.gcMaxAge < now : true);
        }).map(d => d.id);
        if (outdatedMatchIds.length) {
          this.__store.setState(s => {
            const matchesById = {
              ...s.matchesById
            };
            outdatedMatchIds.forEach(id => {
              delete matchesById[id];
            });
            return {
              ...s,
              matchesById
            };
          });
        }
      };
      matchRoutes = (pathname, locationSearch, opts) => {
        let routeParams = {};
        let foundRoute = this.flatRoutes.find(route => {
          const matchedParams = matchPathname(this.basepath, trimPathRight(pathname), {
            to: route.fullPath,
            caseSensitive: route.options.caseSensitive ?? this.options.caseSensitive
          });
          if (matchedParams) {
            routeParams = matchedParams;
            return true;
          }
          return false;
        });
        let routeCursor = foundRoute || this.routesById['__root__'];
        let matchedRoutes = [routeCursor];
        while (routeCursor?.parentRoute) {
          routeCursor = routeCursor.parentRoute;
          if (routeCursor) matchedRoutes.unshift(routeCursor);
        }

        // Alright, by now we should have all of our
        // matching routes and their param pairs, let's
        // Turn them into actual `Match` objects and
        // accumulate the params into a single params bag
        let allParams = {};

        // Existing matches are matches that are already loaded along with
        // pending matches that are still loading

        const matches = matchedRoutes.map(route => {
          let parsedParams;
          let parsedParamsError;
          try {
            parsedParams = route.options.parseParams?.(routeParams) ?? routeParams;
            // (typeof route.options.parseParams === 'object' &&
            // route.options.parseParams.parse
            //   ? route.options.parseParams.parse(routeParams)
            //   : (route.options.parseParams as any)?.(routeParams!)) ?? routeParams
          } catch (err) {
            parsedParamsError = new PathParamError(err.message, {
              cause: err
            });
            if (opts?.throwOnError) {
              throw parsedParamsError;
            }
          }

          // Add the parsed params to the accumulated params bag
          Object.assign(allParams, parsedParams);
          const interpolatedPath = interpolatePath(route.path, allParams);
          const key = route.options.key ? route.options.key({
            params: allParams,
            search: locationSearch
          }) ?? '' : '';
          const stringifiedKey = key ? JSON.stringify(key) : '';
          const matchId = interpolatePath(route.id, allParams, true) + stringifiedKey;

          // Waste not, want not. If we already have a match for this route,
          // reuse it. This is important for layout routes, which might stick
          // around between navigation actions that only change leaf routes.
          const existingMatch = this.getRouteMatch(matchId);
          if (existingMatch) {
            return {
              ...existingMatch
            };
          }

          // Create a fresh route match
          const hasLoaders = !!(route.options.loader || componentTypes.some(d => route.options[d]?.preload));
          const routeMatch = {
            id: matchId,
            key: stringifiedKey,
            routeId: route.id,
            params: allParams,
            pathname: joinPaths([this.basepath, interpolatedPath]),
            updatedAt: Date.now(),
            invalidAt: Infinity,
            preloadInvalidAt: Infinity,
            routeSearch: {},
            search: {},
            status: hasLoaders ? 'idle' : 'success',
            isFetching: false,
            invalid: false,
            error: undefined,
            paramsError: parsedParamsError,
            searchError: undefined,
            loaderData: undefined,
            loadPromise: Promise.resolve(),
            routeContext: undefined,
            context: undefined,
            abortController: new AbortController(),
            fetchedAt: 0
          };
          return routeMatch;
        });

        // Take each match and resolve its search params and context
        // This has to happen after the matches are created or found
        // so that we can use the parent match's search params and context
        matches.forEach((match, i) => {
          const parentMatch = matches[i - 1];
          const route = this.getRoute(match.routeId);
          const searchInfo = (() => {
            // Validate the search params and stabilize them
            const parentSearchInfo = {
              search: parentMatch?.search ?? locationSearch,
              routeSearch: parentMatch?.routeSearch ?? locationSearch
            };
            try {
              const validator = typeof route.options.validateSearch === 'object' ? route.options.validateSearch.parse : route.options.validateSearch;
              const routeSearch = validator?.(parentSearchInfo.search) ?? {};
              const search = {
                ...parentSearchInfo.search,
                ...routeSearch
              };
              return {
                routeSearch: replaceEqualDeep(match.routeSearch, routeSearch),
                search: replaceEqualDeep(match.search, search)
              };
            } catch (err) {
              match.searchError = new SearchParamError(err.message, {
                cause: err
              });
              if (opts?.throwOnError) {
                throw match.searchError;
              }
              return parentSearchInfo;
            }
          })();
          const contextInfo = (() => {
            try {
              const routeContext = route.options.getContext?.({
                parentContext: parentMatch?.routeContext ?? {},
                context: parentMatch?.context ?? this?.options.context ?? {},
                params: match.params,
                search: match.search
              }) || {};
              const context = {
                ...(parentMatch?.context ?? this?.options.context),
                ...routeContext
              };
              return {
                context,
                routeContext
              };
            } catch (err) {
              route.options.onError?.(err);
              throw err;
            }
          })();
          Object.assign(match, {
            ...searchInfo,
            ...contextInfo
          });
        });
        return matches;
      };
      loadMatches = async (resolvedMatches, opts) => {
        this.cleanMatches();
        if (!opts?.preload) {
          resolvedMatches.forEach(match => {
            // Update each match with its latest url data
            this.setRouteMatch(match.id, s => ({
              ...s,
              routeSearch: match.routeSearch,
              search: match.search,
              routeContext: match.routeContext,
              context: match.context,
              error: match.error,
              paramsError: match.paramsError,
              searchError: match.searchError,
              params: match.params
            }));
          });
        }
        let firstBadMatchIndex;

        // Check each match middleware to see if the route can be accessed
        try {
          for (const [index, match] of resolvedMatches.entries()) {
            const route = this.getRoute(match.routeId);
            const handleError = (err, handler) => {
              firstBadMatchIndex = firstBadMatchIndex ?? index;
              handler = handler || route.options.onError;
              if (isRedirect(err)) {
                throw err;
              }
              try {
                handler?.(err);
              } catch (errorHandlerErr) {
                err = errorHandlerErr;
                if (isRedirect(errorHandlerErr)) {
                  throw errorHandlerErr;
                }
              }
              this.setRouteMatch(match.id, s => ({
                ...s,
                error: err,
                status: 'error',
                updatedAt: Date.now()
              }));
            };
            if (match.paramsError) {
              handleError(match.paramsError, route.options.onParseParamsError);
            }
            if (match.searchError) {
              handleError(match.searchError, route.options.onValidateSearchError);
            }
            let didError = false;
            try {
              await route.options.beforeLoad?.({
                ...match,
                preload: !!opts?.preload
              });
            } catch (err) {
              handleError(err, route.options.onBeforeLoadError);
              didError = true;
            }

            // If we errored, do not run the next matches' middleware
            if (didError) {
              break;
            }
          }
        } catch (err) {
          if (!opts?.preload) {
            this.navigate(err);
          }
          throw err;
        }
        const validResolvedMatches = resolvedMatches.slice(0, firstBadMatchIndex);
        const matchPromises = [];
        validResolvedMatches.forEach((match, index) => {
          matchPromises.push((async () => {
            const parentMatchPromise = matchPromises[index - 1];
            const route = this.getRoute(match.routeId);
            if (match.isFetching || match.status === 'success' && !this.getIsInvalid({
              matchId: match.id,
              preload: opts?.preload
            })) {
              return this.getRouteMatch(match.id)?.loadPromise;
            }
            const fetchedAt = Date.now();
            const checkLatest = () => {
              const latest = this.getRouteMatch(match.id);
              return latest && latest.fetchedAt !== fetchedAt ? latest.loadPromise : undefined;
            };
            const loadPromise = (async () => {
              let latestPromise;
              const componentsPromise = Promise.all(componentTypes.map(async type => {
                const component = route.options[type];
                if (component?.preload) {
                  await component.preload();
                }
              }));
              const loaderPromise = route.options.loader?.({
                ...match,
                preload: !!opts?.preload,
                parentMatchPromise
              });
              const handleError = err => {
                if (isRedirect(err)) {
                  if (!opts?.preload) {
                    this.navigate(err);
                  }
                  return true;
                }
                return false;
              };
              try {
                const [_, loader] = await Promise.all([componentsPromise, loaderPromise]);
                if (latestPromise = checkLatest()) return await latestPromise;
                this.setRouteMatchData(match.id, () => loader, opts);
              } catch (err) {
                if (latestPromise = checkLatest()) return await latestPromise;
                if (handleError(err)) {
                  return;
                }
                const errorHandler = route.options.onLoadError ?? route.options.onError;
                let caughtError = err;
                try {
                  errorHandler?.(err);
                } catch (errorHandlerErr) {
                  caughtError = errorHandlerErr;
                  if (handleError(errorHandlerErr)) {
                    return;
                  }
                }
                this.setRouteMatch(match.id, s => ({
                  ...s,
                  error: caughtError,
                  status: 'error',
                  isFetching: false,
                  updatedAt: Date.now()
                }));
              }
            })();
            this.setRouteMatch(match.id, s => ({
              ...s,
              status: s.status !== 'success' ? 'pending' : s.status,
              isFetching: true,
              loadPromise,
              fetchedAt,
              invalid: false
            }));
            await loadPromise;
          })());
        });
        await Promise.all(matchPromises);
      };
      reload = () => {
        return this.navigate({
          fromCurrent: true,
          replace: true,
          search: true
        });
      };
      resolvePath = (from, path) => {
        return resolvePath(this.basepath, from, cleanPath(path));
      };
      navigate = async ({
        from,
        to = '',
        search,
        hash,
        replace,
        params
      }) => {
        // If this link simply reloads the current route,
        // make sure it has a new key so it will trigger a data refresh

        // If this `to` is a valid external URL, return
        // null for LinkUtils
        const toString = String(to);
        const fromString = typeof from === 'undefined' ? from : String(from);
        let isExternal;
        try {
          new URL(`${toString}`);
          isExternal = true;
        } catch (e) {}
        invariant(!isExternal, 'Attempting to navigate to external url with this.navigate!');
        return this.#commitLocation({
          from: fromString,
          to: toString,
          search,
          hash,
          replace,
          params
        });
      };
      matchRoute = (location, opts) => {
        location = {
          ...location,
          to: location.to ? this.resolvePath(location.from ?? '', location.to) : undefined
        };
        const next = this.buildNext(location);
        if (opts?.pending && this.state.status !== 'pending') {
          return false;
        }
        const baseLocation = opts?.pending ? this.state.location : this.state.resolvedLocation;
        if (!baseLocation) {
          return false;
        }
        const match = matchPathname(this.basepath, baseLocation.pathname, {
          ...opts,
          to: next.pathname
        });
        if (!match) {
          return false;
        }
        if (opts?.includeSearch ?? true) {
          return partialDeepEqual(baseLocation.search, next.search) ? match : false;
        }
        return match;
      };
      buildLink = ({
        from,
        to = '.',
        search,
        params,
        hash,
        target,
        replace,
        activeOptions,
        preload,
        preloadDelay: userPreloadDelay,
        disabled
      }) => {
        // If this link simply reloads the current route,
        // make sure it has a new key so it will trigger a data refresh

        // If this `to` is a valid external URL, return
        // null for LinkUtils

        try {
          new URL(`${to}`);
          return {
            type: 'external',
            href: to
          };
        } catch (e) {}
        const nextOpts = {
          from,
          to,
          search,
          params,
          hash,
          replace
        };
        const next = this.buildNext(nextOpts);
        preload = preload ?? this.options.defaultPreload;
        const preloadDelay = userPreloadDelay ?? this.options.defaultPreloadDelay ?? 0;

        // Compare path/hash for matches
        const currentPathSplit = this.state.location.pathname.split('/');
        const nextPathSplit = next.pathname.split('/');
        const pathIsFuzzyEqual = nextPathSplit.every((d, i) => d === currentPathSplit[i]);
        // Combine the matches based on user options
        const pathTest = activeOptions?.exact ? this.state.location.pathname === next.pathname : pathIsFuzzyEqual;
        const hashTest = activeOptions?.includeHash ? this.state.location.hash === next.hash : true;
        const searchTest = activeOptions?.includeSearch ?? true ? partialDeepEqual(this.state.location.search, next.search) : true;

        // The final "active" test
        const isActive = pathTest && hashTest && searchTest;

        // The click handler
        const handleClick = e => {
          if (!disabled && !isCtrlEvent(e) && !e.defaultPrevented && (!target || target === '_self') && e.button === 0) {
            e.preventDefault();

            // All is well? Navigate!
            this.#commitLocation(nextOpts);
          }
        };

        // The click handler
        const handleFocus = e => {
          if (preload) {
            this.preloadRoute(nextOpts).catch(err => {
              console.warn(err);
              console.warn('Error preloading route! ☝️');
            });
          }
        };
        const handleTouchStart = e => {
          this.preloadRoute(nextOpts).catch(err => {
            console.warn(err);
            console.warn('Error preloading route! ☝️');
          });
        };
        const handleEnter = e => {
          const target = e.target || {};
          if (preload) {
            if (target.preloadTimeout) {
              return;
            }
            target.preloadTimeout = setTimeout(() => {
              target.preloadTimeout = null;
              this.preloadRoute(nextOpts).catch(err => {
                console.warn(err);
                console.warn('Error preloading route! ☝️');
              });
            }, preloadDelay);
          }
        };
        const handleLeave = e => {
          const target = e.target || {};
          if (target.preloadTimeout) {
            clearTimeout(target.preloadTimeout);
            target.preloadTimeout = null;
          }
        };
        return {
          type: 'internal',
          next,
          handleFocus,
          handleClick,
          handleEnter,
          handleLeave,
          handleTouchStart,
          isActive,
          disabled
        };
      };
      dehydrate = () => {
        return {
          state: pick(this.state, ['location', 'status', 'lastUpdated'])
        };
      };
      hydrate = async __do_not_use_server_ctx => {
        let _ctx = __do_not_use_server_ctx;
        // Client hydrates from window
        if (typeof document !== 'undefined') {
          _ctx = window.__TSR_DEHYDRATED__;
        }
        invariant(_ctx, 'Expected to find a __TSR_DEHYDRATED__ property on window... but we did not. Did you forget to render <DehydrateRouter /> in your app?');
        const ctx = _ctx;
        this.dehydratedData = ctx.payload;
        this.options.hydrate?.(ctx.payload);
        this.__store.setState(s => {
          return {
            ...s,
            ...ctx.router.state,
            resolvedLocation: ctx.router.state.location
          };
        });
        await this.load();
        return;
      };
      injectedHtml = [];
      injectHtml = async html => {
        this.injectedHtml.push(html);
      };
      dehydrateData = (key, getData) => {
        if (typeof document === 'undefined') {
          const strKey = typeof key === 'string' ? key : JSON.stringify(key);
          this.injectHtml(async () => {
            const id = `__TSR_DEHYDRATED__${strKey}`;
            const data = typeof getData === 'function' ? await getData() : getData;
            return `<script id='${id}' suppressHydrationWarning>window["__TSR_DEHYDRATED__${escapeJSON(strKey)}"] = ${JSON.stringify(data)}
        ;(() => {
          var el = document.getElementById('${id}')
          el.parentElement.removeChild(el)
        })()
        </script>`;
          });
          return () => this.hydrateData(key);
        }
        return () => undefined;
      };
      hydrateData = key => {
        if (typeof document !== 'undefined') {
          const strKey = typeof key === 'string' ? key : JSON.stringify(key);
          return window[`__TSR_DEHYDRATED__${strKey}`];
        }
        return undefined;
      };

      // resolveMatchPromise = (matchId: string, key: string, value: any) => {
      //   this.state.matches
      //     .find((d) => d.id === matchId)
      //     ?.__promisesByKey[key]?.resolve(value)
      // }

      #buildRouteTree = routeTree => {
        this.routeTree = routeTree;
        this.routesById = {};
        this.routesByPath = {};
        this.flatRoutes = [];
        const recurseRoutes = routes => {
          routes.forEach((route, i) => {
            route.init({
              originalIndex: i,
              router: this
            });
            const existingRoute = this.routesById[route.id];
            invariant(!existingRoute, `Duplicate routes found with id: ${String(route.id)}`);
            this.routesById[route.id] = route;
            if (!route.isRoot && route.path) {
              const trimmedFullPath = trimPathRight(route.fullPath);
              if (!this.routesByPath[trimmedFullPath] || route.fullPath.endsWith('/')) {
                this.routesByPath[trimmedFullPath] = route;
              }
            }
            const children = route.children;
            if (children?.length) {
              recurseRoutes(children);
            }
          });
        };
        recurseRoutes([routeTree]);
        this.flatRoutes = Object.values(this.routesByPath).map((d, i) => {
          const trimmed = trimPath(d.fullPath);
          const parsed = parsePathname(trimmed);
          while (parsed.length > 1 && parsed[0]?.value === '/') {
            parsed.shift();
          }
          const score = parsed.map(d => {
            if (d.type === 'param') {
              return 0.5;
            }
            if (d.type === 'wildcard') {
              return 0.25;
            }
            return 1;
          });
          return {
            child: d,
            trimmed,
            parsed,
            index: i,
            score
          };
        }).sort((a, b) => {
          let isIndex = a.trimmed === '/' ? 1 : b.trimmed === '/' ? -1 : 0;
          if (isIndex !== 0) return isIndex;
          const length = Math.min(a.score.length, b.score.length);

          // Sort by length of score
          if (a.score.length !== b.score.length) {
            return b.score.length - a.score.length;
          }

          // Sort by min available score
          for (let i = 0; i < length; i++) {
            if (a.score[i] !== b.score[i]) {
              return b.score[i] - a.score[i];
            }
          }

          // Sort by min available parsed value
          for (let i = 0; i < length; i++) {
            if (a.parsed[i].value !== b.parsed[i].value) {
              return a.parsed[i].value > b.parsed[i].value ? 1 : -1;
            }
          }

          // Sort by length of trimmed full path
          if (a.trimmed !== b.trimmed) {
            return a.trimmed > b.trimmed ? 1 : -1;
          }

          // Sort by original index
          return a.index - b.index;
        }).map((d, i) => {
          d.child.rank = i;
          return d.child;
        });
      };
      #parseLocation = previousLocation => {
        let {
          pathname,
          search,
          hash,
          state
        } = this.history.location;
        const parsedSearch = this.options.parseSearch(search);
        return {
          pathname: pathname,
          searchStr: search,
          search: replaceEqualDeep(previousLocation?.search, parsedSearch),
          hash: hash.split('#').reverse()[0] ?? '',
          href: `${pathname}${search}${hash}`,
          state: state,
          key: state?.key || '__init__'
        };
      };
      #buildLocation = (dest = {}) => {
        dest.fromCurrent = dest.fromCurrent ?? dest.to === '';
        const fromPathname = dest.fromCurrent ? this.state.location.pathname : dest.from ?? this.state.location.pathname;
        let pathname = resolvePath(this.basepath ?? '/', fromPathname, `${dest.to ?? ''}`);
        const fromMatches = this.matchRoutes(this.state.location.pathname, this.state.location.search);
        const prevParams = {
          ...last(fromMatches)?.params
        };
        let nextParams = (dest.params ?? true) === true ? prevParams : functionalUpdate(dest.params, prevParams);
        if (nextParams) {
          dest.__matches?.map(d => this.getRoute(d.routeId).options.stringifyParams).filter(Boolean).forEach(fn => {
            nextParams = {
              ...nextParams,
              ...fn(nextParams)
            };
          });
        }
        pathname = interpolatePath(pathname, nextParams ?? {});
        const preSearchFilters = dest.__matches?.map(match => this.getRoute(match.routeId).options.preSearchFilters ?? []).flat().filter(Boolean) ?? [];
        const postSearchFilters = dest.__matches?.map(match => this.getRoute(match.routeId).options.postSearchFilters ?? []).flat().filter(Boolean) ?? [];

        // Pre filters first
        const preFilteredSearch = preSearchFilters?.length ? preSearchFilters?.reduce((prev, next) => next(prev), this.state.location.search) : this.state.location.search;

        // Then the link/navigate function
        const destSearch = dest.search === true ? preFilteredSearch // Preserve resolvedFrom true
        : dest.search ? functionalUpdate(dest.search, preFilteredSearch) ?? {} // Updater
        : preSearchFilters?.length ? preFilteredSearch // Preserve resolvedFrom filters
        : {};

        // Then post filters
        const postFilteredSearch = postSearchFilters?.length ? postSearchFilters.reduce((prev, next) => next(prev), destSearch) : destSearch;
        const search = replaceEqualDeep(this.state.location.search, postFilteredSearch);
        const searchStr = this.options.stringifySearch(search);
        const hash = dest.hash === true ? this.state.location.hash : functionalUpdate(dest.hash, this.state.location.hash);
        const hashStr = hash ? `#${hash}` : '';
        const nextState = dest.state === true ? this.state.location.state : functionalUpdate(dest.state, this.state.location.state);
        return {
          pathname,
          search,
          searchStr,
          state: nextState,
          hash,
          href: this.history.createHref(`${pathname}${searchStr}${hashStr}`),
          key: dest.key
        };
      };
      #commitLocation = async location => {
        const next = this.buildNext(location);
        const id = '' + Date.now() + Math.random();
        if (this.navigateTimeout) clearTimeout(this.navigateTimeout);
        let nextAction = 'replace';
        if (!location.replace) {
          nextAction = 'push';
        }
        const isSameUrl = this.state.location.href === next.href;
        if (isSameUrl && !next.key) {
          nextAction = 'replace';
        }
        const href = `${next.pathname}${next.searchStr}${next.hash ? `#${next.hash}` : ''}`;
        this.history[nextAction === 'push' ? 'push' : 'replace'](href, {
          id,
          ...next.state
        });
        return this.latestLoadPromise;
      };
      getRouteMatch = id => {
        return this.state.matchesById[id];
      };
      setRouteMatch = (id, updater) => {
        this.__store.setState(prev => ({
          ...prev,
          matchesById: {
            ...prev.matchesById,
            [id]: updater(prev.matchesById[id])
          }
        }));
      };
      setRouteMatchData = (id, updater, opts) => {
        const match = this.getRouteMatch(id);
        if (!match) return;
        const route = this.getRoute(match.routeId);
        const updatedAt = opts?.updatedAt ?? Date.now();
        const preloadInvalidAt = updatedAt + (opts?.maxAge ?? route.options.preloadMaxAge ?? this.options.defaultPreloadMaxAge ?? 5000);
        const invalidAt = updatedAt + (opts?.maxAge ?? route.options.maxAge ?? this.options.defaultMaxAge ?? Infinity);
        this.setRouteMatch(id, s => ({
          ...s,
          error: undefined,
          status: 'success',
          isFetching: false,
          updatedAt: Date.now(),
          loaderData: functionalUpdate(updater, s.loaderData),
          preloadInvalidAt,
          invalidAt
        }));
        if (this.state.matches.find(d => d.id === id)) ;
      };
      invalidate = async opts => {
        if (opts?.matchId) {
          this.setRouteMatch(opts.matchId, s => ({
            ...s,
            invalid: true
          }));
          const matchIndex = this.state.matches.findIndex(d => d.id === opts.matchId);
          const childMatch = this.state.matches[matchIndex + 1];
          if (childMatch) {
            return this.invalidate({
              matchId: childMatch.id,
              reload: false
            });
          }
        } else {
          this.__store.batch(() => {
            Object.values(this.state.matchesById).forEach(match => {
              this.setRouteMatch(match.id, s => ({
                ...s,
                invalid: true
              }));
            });
          });
        }
        if (opts?.reload ?? true) {
          return this.reload();
        }
      };
      getIsInvalid = opts => {
        if (!opts?.matchId) {
          return !!this.state.matches.find(d => this.getIsInvalid({
            matchId: d.id,
            preload: opts?.preload
          }));
        }
        const match = this.getRouteMatch(opts?.matchId);
        if (!match) {
          return false;
        }
        const now = Date.now();
        return match.invalid || (opts?.preload ? match.preloadInvalidAt : match.invalidAt) < now;
      };
    }

    // Detect if we're in the DOM
    const isServer = typeof window === 'undefined' || !window.document.createElement;
    function getInitialRouterState() {
      return {
        status: 'idle',
        isFetching: false,
        resolvedLocation: null,
        location: null,
        matchesById: {},
        matchIds: [],
        pendingMatchIds: [],
        matches: [],
        pendingMatches: [],
        lastUpdated: Date.now()
      };
    }
    function isCtrlEvent(e) {
      return !!(e.metaKey || e.altKey || e.ctrlKey || e.shiftKey);
    }
    function redirect(opts) {
      opts.isRedirect = true;
      return opts;
    }
    function isRedirect(obj) {
      return !!obj?.isRedirect;
    }
    class SearchParamError extends Error {}
    class PathParamError extends Error {}
    function escapeJSON(jsonString) {
      return jsonString.replace(/\\/g, '\\\\') // Escape backslashes
      .replace(/'/g, "\\'") // Escape single quotes
      .replace(/"/g, '\\"'); // Escape double quotes
    }

    // A function that takes an import() argument which is a function and returns a new function that will
    // proxy arguments from the caller to the imported function, retaining all type
    // information along the way
    function lazyFn(fn, key) {
      return async (...args) => {
        const imported = await fn();
        return imported[key || 'default'](...args);
      };
    }

    exports.FileRoute = FileRoute;
    exports.PathParamError = PathParamError;
    exports.RootRoute = RootRoute;
    exports.Route = Route;
    exports.Router = Router;
    exports.RouterContext = RouterContext;
    exports.SearchParamError = SearchParamError;
    exports.cleanPath = cleanPath;
    exports.componentTypes = componentTypes;
    exports.createBrowserHistory = createBrowserHistory;
    exports.createHashHistory = createHashHistory;
    exports.createMemoryHistory = createMemoryHistory;
    exports.decode = decode;
    exports.defaultParseSearch = defaultParseSearch;
    exports.defaultStringifySearch = defaultStringifySearch;
    exports.encode = encode;
    exports.functionalUpdate = functionalUpdate;
    exports.interpolatePath = interpolatePath;
    exports.invariant = invariant;
    exports.isPlainObject = isPlainObject;
    exports.isRedirect = isRedirect;
    exports.joinPaths = joinPaths;
    exports.last = last;
    exports.lazyFn = lazyFn;
    exports.matchByPath = matchByPath;
    exports.matchPathname = matchPathname;
    exports.parsePathname = parsePathname;
    exports.parseSearchWith = parseSearchWith;
    exports.partialDeepEqual = partialDeepEqual;
    exports.pick = pick;
    exports.redirect = redirect;
    exports.replaceEqualDeep = replaceEqualDeep;
    exports.resolvePath = resolvePath;
    exports.rootRouteId = rootRouteId;
    exports.stringifySearchWith = stringifySearchWith;
    exports.trimPath = trimPath;
    exports.trimPathLeft = trimPathLeft;
    exports.trimPathRight = trimPathRight;
    exports.warning = warning;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.development.js.map
